# Azle/Motoko/Rust Benchmarks

-   These benchmarks should be considered preliminary (especially the Motoko benchmarks, something seems off with the function prelude)
-   These benchmarks were implemented using the performance counter API
    -   Performance counter information in [The Internet Computer Interface Spec](https://internetcomputer.org/docs/current/references/ic-interface-spec/#system-api-imports)
    -   Performance counter information in [the forum](https://forum.dfinity.org/t/introducing-performance-counter-on-the-internet-computer/14027)
-   Each benchmark is the average of 10 runs
-   The following may be inaccurate or missing from the benchmarks as described [here](https://forum.dfinity.org/t/introducing-performance-counter-on-the-internet-computer/14027):
    -   Candid serialization/deserialization of function parameters and return types
    -   Canister method prologue/epilogue
    -   Some Motoko runtime behavior (such as garbage collection)
-   You can find a raw CSV file with this data [here](./benchmarks.csv)
-   A rough USD cost model for various app scenarios can be found [here](https://docs.google.com/spreadsheets/d/1PQ53R9hYE1fuMB_z-Bl6dyymm7end7rVJ85TvGEh0BQ)

The format for benchmark numbers is (x / y) where:

-   x = Wasm instructions counted only in the function body
-   y = Wasm instructions counted in the function body and the function prelude

## USD Cost Estimates Per Year

These estimates use the average Wasm instructions per function call including the function prelude.

The Wasm instruction counts used are:

-   Azle: 6_149_703
-   Motoko: 248_920
-   Rust: 111_714

### Cycle Costs Table

Cycle costs taken from [here](https://internetcomputer.org/docs/current/developer-docs/deploy/computation-and-storage-costs/).

| Compute Percent Allocated Per Second | Update Message Execution | Ten Update Instructions Execution | Xnet Call | Xnet Byte Transmission | Ingress Message Reception | Ingress Byte Reception | GB Storage Per Second |
| ------------------------------------ | ------------------------ | --------------------------------- | --------- | ---------------------- | ------------------------- | ---------------------- | --------------------- |
| 100_000                              | 590_000                  | 4                                 | 260_000   | 1_000                  | 1_200_000                 | 2_000                  | 127_000               |

### Application Scenarios

| Usage    | Query/Update Heaviness | Ingress Bytes Per Query Message | Ingress Bytes Per Update Message | GB Storage | Query Messages Per Second | Update Messages Per Second | Xnet Calls Per Second | Xnet Call Bytes |
| -------- | ---------------------- | ------------------------------- | -------------------------------- | ---------- | ------------------------- | -------------------------- | --------------------- | --------------- |
| Light    | Even                   | 100                             | 100                              | 0.5        | 0.01                      | 0.01                       | 0.001                 | 20              |
| Light    | Query Heavy            | 100                             | 100                              | 0.5        | 0.01                      | 0.0001                     | 0.001                 | 20              |
| Light    | Update Heavy           | 100                             | 100                              | 0.5        | 0.0001                    | 0.01                       | 0.001                 | 20              |
| Moderate | Even                   | 1_000                           | 1_000                            | 1          | 1                         | 1                          | 0.1                   | 200             |
| Moderate | Query Heavy            | 1_000                           | 1_000                            | 1          | 1                         | 0.01                       | 0.1                   | 200             |
| Moderate | Update Heavy           | 1_000                           | 1_000                            | 1          | 0.01                      | 1                          | 0.1                   | 200             |
| Heavy    | Even                   | 10_000                          | 10_000                           | 2          | 100                       | 100                        | 10                    | 2_000           |
| Heavy    | Query Heavy            | 10_000                          | 10_000                           | 2          | 100                       | 1                          | 10                    | 2_000           |
| Heavy    | Update Heavy           | 10_000                          | 10_000                           | 2          | 1                         | 100                        | 10                    | 2_000           |

### Application USD Cost Estimates Per Year

| Usage    | Query/Update Heaviness | CDK    | Ingress Messages | Ingress Bytes Query Messages | Ingress Bytes Update Messages | Update Messages | Update Instructions | Xnet Calls | Xnet Byte Transmission | GB Storage | Total Cost  |
| -------- | ---------------------- | ------ | ---------------- | ---------------------------- | ----------------------------- | --------------- | ------------------- | ---------- | ---------------------- | ---------- | ----------- |
| Light    | Even                   | Azle   | $1.00            | $0.08                        | $0.08                         | $0.25           | $1.02               | $0.01      | $0.00                  | $2.64      | $5.09       |
| Light    | Even                   | Motoko | $1.00            | $0.08                        | $0.08                         | $0.25           | $0.04               | $0.01      | $0.00                  | $2.64      | $4.11       |
| Light    | Even                   | Rust   | $1.00            | $0.08                        | $0.08                         | $0.25           | $0.02               | $0.01      | $0.00                  | $2.64      | $4.08       |
| Light    | Query Heavy            | Azle   | $0.50            | $0.08                        | $0.00                         | $0.00           | $0.01               | $0.01      | $0.00                  | $2.64      | $3.26       |
| Light    | Query Heavy            | Motoko | $0.50            | $0.08                        | $0.00                         | $0.00           | $0.00               | $0.01      | $0.00                  | $2.64      | $3.25       |
| Light    | Query Heavy            | Rust   | $0.50            | $0.08                        | $0.00                         | $0.00           | $0.00               | $0.01      | $0.00                  | $2.64      | $3.25       |
| Light    | Update Heavy           | Azle   | $0.50            | $0.00                        | $0.08                         | $0.25           | $1.02               | $0.01      | $0.00                  | $2.64      | $4.51       |
| Light    | Update Heavy           | Motoko | $0.50            | $0.00                        | $0.08                         | $0.25           | $0.04               | $0.01      | $0.00                  | $2.64      | $3.53       |
| Light    | Update Heavy           | Rust   | $0.50            | $0.00                        | $0.08                         | $0.25           | $0.02               | $0.01      | $0.00                  | $2.64      | $3.51       |
| Moderate | Even                   | Azle   | $99.91           | $83.26                       | $83.26                        | $24.56          | $102.40             | $1.08      | $0.83                  | $5.29      | $400.58     |
| Moderate | Even                   | Motoko | $99.91           | $83.26                       | $83.26                        | $24.56          | $4.14               | $1.08      | $0.83                  | $5.29      | $302.32     |
| Moderate | Even                   | Rust   | $99.91           | $83.26                       | $83.26                        | $24.56          | $1.86               | $1.08      | $0.83                  | $5.29      | $300.04     |
| Moderate | Query Heavy            | Azle   | $50.45           | $83.26                       | $0.83                         | $0.25           | $1.02               | $1.08      | $0.83                  | $5.29      | $143.01     |
| Moderate | Query Heavy            | Motoko | $50.45           | $83.26                       | $0.83                         | $0.25           | $0.04               | $1.08      | $0.83                  | $5.29      | $142.03     |
| Moderate | Query Heavy            | Rust   | $50.45           | $83.26                       | $0.83                         | $0.25           | $0.02               | $1.08      | $0.83                  | $5.29      | $142.01     |
| Moderate | Update Heavy           | Azle   | $50.45           | $0.83                        | $83.26                        | $24.56          | $102.40             | $1.08      | $0.83                  | $5.29      | $268.70     |
| Moderate | Update Heavy           | Motoko | $50.45           | $0.83                        | $83.26                        | $24.56          | $4.14               | $1.08      | $0.83                  | $5.29      | $170.45     |
| Moderate | Update Heavy           | Rust   | $50.45           | $0.83                        | $83.26                        | $24.56          | $1.86               | $1.08      | $0.83                  | $5.29      | $168.16     |
| Heavy    | Even                   | Azle   | $9,990.60        | $83,255.04                   | $83,255.04                    | $2,456.02       | $10,239.87          | $108.23    | $832.55                | $10.57     | $190,147.94 |
| Heavy    | Even                   | Motoko | $9,990.60        | $83,255.04                   | $83,255.04                    | $2,456.02       | $414.48             | $108.23    | $832.55                | $10.57     | $180,322.54 |
| Heavy    | Even                   | Rust   | $9,990.60        | $83,255.04                   | $83,255.04                    | $2,456.02       | $186.01             | $108.23    | $832.55                | $10.57     | $180,094.08 |
| Heavy    | Query Heavy            | Azle   | $5,045.26        | $83,255.04                   | $832.55                       | $24.56          | $102.40             | $108.23    | $832.55                | $10.57     | $90,211.16  |
| Heavy    | Query Heavy            | Motoko | $5,045.26        | $83,255.04                   | $832.55                       | $24.56          | $4.14               | $108.23    | $832.55                | $10.57     | $90,112.91  |
| Heavy    | Query Heavy            | Rust   | $5,045.26        | $83,255.04                   | $832.55                       | $24.56          | $1.86               | $108.23    | $832.55                | $10.57     | $90,110.62  |
| Heavy    | Update Heavy           | Azle   | $5,045.26        | $832.55                      | $83,255.04                    | $2,456.02       | $10,239.87          | $108.23    | $832.55                | $10.57     | $102,780.10 |
| Heavy    | Update Heavy           | Motoko | $5,045.26        | $832.55                      | $83,255.04                    | $2,456.02       | $414.48             | $108.23    | $832.55                | $10.57     | $92,954.70  |
| Heavy    | Update Heavy           | Rust   | $5,045.26        | $832.55                      | $83,255.04                    | $2,456.02       | $186.01             | $108.23    | $832.55                | $10.57     | $92,726.24  |

## Averages

| Azle Wasm Instructions  | Motoko Wasm Instructions | Rust Wasm Instructions | Azle/Motoko Change Multiplier | Azle/Rust Change Multiplier | Motoko/Azle Change Multiplier | Motoko/Rust Change Multiplier | Rust/Azle Change Multiplier | Rust/Motoko Change Multiplier |
| ----------------------- | ------------------------ | ---------------------- | ----------------------------- | --------------------------- | ----------------------------- | ----------------------------- | --------------------------- | ----------------------------- |
| (3_231_009 / 6_149_703) | (246_375 / 248_920)      | (85_335 / 111_714)     | (1_028x / 623x)               | (1_405x / 119x)             | (-1_028x / -623x)             | (2x / -3x)                    | (-1_405x / -119x)           | (-2x / 3x)                    |

## Benchmarks

| Description               | Azle Wasm Instructions    | Motoko Wasm Instructions | Rust Wasm Instructions  | Azle/Motoko Change Multiplier | Azle/Rust Change Multiplier | Motoko/Azle Change Multiplier | Motoko/Rust Change Multiplier | Rust/Azle Change Multiplier | Rust/Motoko Change Multiplier |
| ------------------------- | ------------------------- | ------------------------ | ----------------------- | ----------------------------- | --------------------------- | ----------------------------- | ----------------------------- | --------------------------- | ----------------------------- |
| empty                     | (1_439_106 / 4_133_864)   | (200 / 2_432)            | (206 / 13_218)          | (7_196x / 1_700x)             | (6_986x / 313x)             | (-7_196x / -1_700x)           | (-1x / -5x)                   | (-6_986x / -313x)           | (1x / 5x)                     |
| blob_init_stack: 1        | (1_481_627 / 4_376_699)   | (785 / 3_333)            | (478 / 27_190)          | (1_887x / 1_313x)             | (3_103x / 162x)             | (-1_887x / -1_313x)           | (2x / -8x)                    | (-3_103x / -162x)           | (-2x / 8x)                    |
| blob_init_stack: 10       | (1_697_250 / 4_605_687)   | (3_611 / 6_159)          | (1_679 / 27_815)        | (470x / 748x)                 | (1_012x / 166x)             | (-470x / -748x)               | (2x / -5x)                    | (-1_012x / -166x)           | (-2x / 5x)                    |
| blob_init_stack: 100      | (3_847_210 / 6_766_531)   | (33_186 / 35_734)        | (14_648 / 40_784)       | (116x / 189x)                 | (263x / 166x)               | (-116x / -189x)               | (2x / -1x)                    | (-263x / -166x)             | (-2x / 1x)                    |
| blob_init_heap: 1         | (1_499_369 / 4_421_470)   | (5_664 / 8_212)          | (2_625 / 28_821)        | (265x / 538x)                 | (572x / 153x)               | (-265x / -538x)               | (2x / -4x)                    | (-572x / -153x)             | (-2x / 4x)                    |
| blob_init_heap: 10        | (1_750_533 / 4_665_970)   | (54_921 / 57_469)        | (22_748 / 48_947)       | (32x / 81x)                   | (77x / 95x)                 | (-32x / -81x)                 | (2x / 1x)                     | (-77x / -95x)               | (-2x / -1x)                   |
| blob_init_heap: 100       | (4_372_700 / 7_269_034)   | (1_353_646 / 1_356_194)  | (230_203 / 256_365)     | (3x / 5x)                     | (19x / 28x)                 | (-3x / -5x)                   | (6x / 5x)                     | (-19x / -28x)               | (-6x / -5x)                   |
| boolean_init_stack: 1     | (1_453_826 / 4_358_877)   | (342 / 2_890)            | (255 / 26_409)          | (4_251x / 1_508x)             | (5_701x / 165x)             | (-4_251x / -1_508x)           | (1x / -9x)                    | (-5_701x / -165x)           | (-1x / 9x)                    |
| boolean_init_stack: 10    | (1_470_976 / 4_387_798)   | (846 / 3_394)            | (417 / 26_571)          | (1_739x / 1_293x)             | (3_528x / 165x)             | (-1_739x / -1_293x)           | (2x / -8x)                    | (-3_528x / -165x)           | (-2x / 8x)                    |
| boolean_init_stack: 100   | (1_585_756 / 4_505_845)   | (5_886 / 8_434)          | (2_037 / 28_191)        | (269x / 534x)                 | (778x / 160x)               | (-269x / -534x)               | (3x / -3x)                    | (-778x / -160x)             | (-3x / 3x)                    |
| boolean_init_heap: 1      | (1_466_180 / 4_387_659)   | (5_186 / 7_734)          | (2_371 / 28_505)        | (283x / 567x)                 | (622x / 154x)               | (-283x / -567x)               | (2x / -4x)                    | (-622x / -154x)             | (-2x / 4x)                    |
| boolean_init_heap: 10     | (1_524_551 / 4_443_457)   | (51_804 / 54_352)        | (21_020 / 47_155)       | (29x / 82x)                   | (73x / 94x)                 | (-29x / -82x)                 | (2x / 1x)                     | (-73x / -94x)               | (-2x / -1x)                   |
| boolean_init_heap: 100    | (2_106_382 / 5_029_155)   | (1_322_021 / 1_324_569)  | (213_771 / 239_932)     | (2x / 4x)                     | (10x / 21x)                 | (-2x / -4x)                   | (6x / 6x)                     | (-10x / -21x)               | (-6x / -6x)                   |
| float32_init_stack: 1     | (1_462_372 / 4_384_726)   | (342 / 2_890)            | (255 / 26_408)          | (4_276x / 1_517x)             | (5_735x / 166x)             | (-4_276x / -1_517x)           | (1x / -9x)                    | (-5_735x / -166x)           | (-1x / 9x)                    |
| float32_init_stack: 10    | (1_476_988 / 4_400_077)   | (846 / 3_394)            | (417 / 26_570)          | (1_746x / 1_296x)             | (3_542x / 166x)             | (-1_746x / -1_296x)           | (2x / -8x)                    | (-3_542x / -166x)           | (-2x / 8x)                    |
| float32_init_stack: 100   | (1_617_525 / 4_540_654)   | (5_886 / 8_434)          | (2_037 / 28_190)        | (275x / 538x)                 | (794x / 161x)               | (-275x / -538x)               | (3x / -3x)                    | (-794x / -161x)             | (-3x / 3x)                    |
| float32_init_heap: 1      | (1_467_998 / 4_392_040)   | (5_253 / 7_801)          | (2_305 / 28_464)        | (280x / 563x)                 | (638x / 154x)               | (-280x / -563x)               | (2x / -4x)                    | (-638x / -154x)             | (-2x / 4x)                    |
| float32_init_heap: 10     | (1_527_865 / 4_447_596)   | (52_517 / 55_065)        | (21_054 / 47_254)       | (29x / 81x)                   | (73x / 94x)                 | (-29x / -81x)                 | (3x / 1x)                     | (-73x / -94x)               | (-3x / -1x)                   |
| float32_init_heap: 100    | (2_141_512 / 5_063_183)   | (1_329_225 / 1_331_773)  | (213_830 / 240_019)     | (2x / 4x)                     | (10x / 21x)                 | (-2x / -4x)                   | (6x / 6x)                     | (-10x / -21x)               | (-6x / -6x)                   |
| float64_init_stack: 1     | (1_462_170 / 4_385_191)   | (342 / 2_890)            | (253 / 26_434)          | (4_275x / 1_517x)             | (5_779x / 166x)             | (-4_275x / -1_517x)           | (1x / -9x)                    | (-5_779x / -166x)           | (-1x / 9x)                    |
| float64_init_stack: 10    | (1_477_220 / 4_399_465)   | (846 / 3_394)            | (397 / 26_578)          | (1_746x / 1_296x)             | (3_721x / 166x)             | (-1_746x / -1_296x)           | (2x / -8x)                    | (-3_721x / -166x)           | (-2x / 8x)                    |
| float64_init_stack: 100   | (1_618_923 / 4_542_916)   | (5_886 / 8_434)          | (1_837 / 28_018)        | (275x / 539x)                 | (881x / 162x)               | (-275x / -539x)               | (3x / -3x)                    | (-881x / -162x)             | (-3x / 3x)                    |
| float64_init_heap: 1      | (1_468_052 / 4_389_797)   | (5_256 / 7_804)          | (2_308 / 28_495)        | (279x / 563x)                 | (638x / 154x)               | (-279x / -563x)               | (2x / -4x)                    | (-638x / -154x)             | (-2x / 4x)                    |
| float64_init_heap: 10     | (1_529_319 / 4_453_785)   | (52_926 / 55_474)        | (21_326 / 47_533)       | (29x / 80x)                   | (72x / 94x)                 | (-29x / -80x)                 | (3x / 1x)                     | (-72x / -94x)               | (-3x / -1x)                   |
| float64_init_heap: 100    | (2_143_335 / 5_062_371)   | (1_282_577 / 1_285_125)  | (217_574 / 243_970)     | (2x / 4x)                     | (10x / 21x)                 | (-2x / -4x)                   | (6x / 5x)                     | (-10x / -21x)               | (-6x / -5x)                   |
| int_init_stack: 1         | (1_461_447 / 4_383_489)   | (342 / 2_890)            | (917 / 27_330)          | (4_273x / 1_517x)             | (1_594x / 160x)             | (-4_273x / -1_517x)           | (-3x / -9x)                   | (-1_594x / -160x)           | (3x / 9x)                     |
| int_init_stack: 10        | (1_472_281 / 4_393_365)   | (846 / 3_394)            | (3_863 / 30_276)        | (1_740x / 1_294x)             | (381x / 145x)               | (-1_740x / -1_294x)           | (-5x / -9x)                   | (-381x / -145x)             | (5x / 9x)                     |
| int_init_stack: 100       | (1_591_026 / 4_515_531)   | (5_886 / 8_434)          | (36_488 / 62_901)       | (270x / 535x)                 | (44x / 72x)                 | (-270x / -535x)               | (-6x / -7x)                   | (-44x / -72x)               | (6x / 7x)                     |
| int_init_heap: 1          | (1_467_721 / 4_389_126)   | (5_221 / 7_769)          | (3_107 / 29_432)        | (281x / 565x)                 | (473x / 149x)               | (-281x / -565x)               | (2x / -4x)                    | (-473x / -149x)             | (-2x / 4x)                    |
| int_init_heap: 10         | (1_527_354 / 4_450_429)   | (51_945 / 54_493)        | (25_227 / 51_545)       | (29x / 82x)                   | (61x / 86x)                 | (-29x / -82x)                 | (2x / 1x)                     | (-61x / -86x)               | (-2x / -1x)                   |
| int_init_heap: 100        | (2_126_355 / 5_048_349)   | (1_323_563 / 1_326_111)  | (254_753 / 280_984)     | (2x / 4x)                     | (8x / 18x)                  | (-2x / -4x)                   | (5x / 5x)                     | (-8x / -18x)                | (-5x / -5x)                   |
| int8_init_stack: 1        | (1_463_253 / 4_384_565)   | (342 / 2_890)            | (255 / 26_426)          | (4_279x / 1_517x)             | (5_738x / 166x)             | (-4_279x / -1_517x)           | (1x / -9x)                    | (-5_738x / -166x)           | (-1x / 9x)                    |
| int8_init_stack: 10       | (1_472_372 / 4_393_511)   | (846 / 3_394)            | (417 / 26_588)          | (1_740x / 1_294x)             | (3_531x / 165x)             | (-1_740x / -1_294x)           | (2x / -8x)                    | (-3_531x / -165x)           | (-2x / 8x)                    |
| int8_init_stack: 100      | (1_588_870 / 4_511_200)   | (5_886 / 8_434)          | (2_037 / 28_208)        | (270x / 535x)                 | (780x / 160x)               | (-270x / -535x)               | (3x / -3x)                    | (-780x / -160x)             | (-3x / 3x)                    |
| int8_init_heap: 1         | (1_468_238 / 4_390_458)   | (5_171 / 7_719)          | (2_358 / 28_590)        | (284x / 569x)                 | (626x / 154x)               | (-284x / -569x)               | (2x / -4x)                    | (-626x / -154x)             | (-2x / 4x)                    |
| int8_init_heap: 10        | (1_525_601 / 4_449_272)   | (51_702 / 54_250)        | (21_256 / 47_476)       | (30x / 82x)                   | (72x / 94x)                 | (-30x / -82x)                 | (2x / 1x)                     | (-72x / -94x)               | (-2x / -1x)                   |
| int8_init_heap: 100       | (2_117_107 / 5_038_086)   | (1_321_084 / 1_323_632)  | (216_421 / 242_631)     | (2x / 4x)                     | (10x / 21x)                 | (-2x / -4x)                   | (6x / 5x)                     | (-10x / -21x)               | (-6x / -5x)                   |
| int16_init_stack: 1       | (1_461_694 / 4_386_733)   | (342 / 2_890)            | (255 / 26_458)          | (4_274x / 1_518x)             | (5_732x / 166x)             | (-4_274x / -1_518x)           | (1x / -9x)                    | (-5_732x / -166x)           | (-1x / 9x)                    |
| int16_init_stack: 10      | (1_472_119 / 4_393_334)   | (846 / 3_394)            | (417 / 26_620)          | (1_740x / 1_294x)             | (3_530x / 165x)             | (-1_740x / -1_294x)           | (2x / -8x)                    | (-3_530x / -165x)           | (-2x / 8x)                    |
| int16_init_stack: 100     | (1_586_194 / 4_510_658)   | (5_886 / 8_434)          | (2_037 / 28_240)        | (269x / 535x)                 | (779x / 160x)               | (-269x / -535x)               | (3x / -3x)                    | (-779x / -160x)             | (-3x / 3x)                    |
| int16_init_heap: 1        | (1_468_587 / 4_392_737)   | (5_171 / 7_719)          | (2_358 / 28_567)        | (284x / 569x)                 | (626x / 154x)               | (-284x / -569x)               | (2x / -4x)                    | (-626x / -154x)             | (-2x / 4x)                    |
| int16_init_heap: 10       | (1_527_509 / 4_448_011)   | (51_696 / 54_244)        | (21_298 / 47_462)       | (30x / 82x)                   | (72x / 94x)                 | (-30x / -82x)                 | (2x / 1x)                     | (-72x / -94x)               | (-2x / -1x)                   |
| int16_init_heap: 100      | (2_118_685 / 5_039_506)   | (1_321_090 / 1_323_638)  | (216_570 / 242_774)     | (2x / 4x)                     | (10x / 21x)                 | (-2x / -4x)                   | (6x / 5x)                     | (-10x / -21x)               | (-6x / -5x)                   |
| int32_init_stack: 1       | (1_462_285 / 4_387_148)   | (342 / 2_890)            | (255 / 26_458)          | (4_276x / 1_518x)             | (5_734x / 166x)             | (-4_276x / -1_518x)           | (1x / -9x)                    | (-5_734x / -166x)           | (-1x / 9x)                    |
| int32_init_stack: 10      | (1_475_400 / 4_397_500)   | (846 / 3_394)            | (417 / 26_620)          | (1_744x / 1_296x)             | (3_538x / 165x)             | (-1_744x / -1_296x)           | (2x / -8x)                    | (-3_538x / -165x)           | (-2x / 8x)                    |
| int32_init_stack: 100     | (1_587_829 / 4_512_351)   | (5_886 / 8_434)          | (2_037 / 28_240)        | (270x / 535x)                 | (779x / 160x)               | (-270x / -535x)               | (3x / -3x)                    | (-779x / -160x)             | (-3x / 3x)                    |
| int32_init_heap: 1        | (1_466_167 / 4_386_897)   | (5_260 / 7_808)          | (2_356 / 28_562)        | (279x / 562x)                 | (626x / 154x)               | (-279x / -562x)               | (2x / -4x)                    | (-626x / -154x)             | (-2x / 4x)                    |
| int32_init_heap: 10       | (1_525_720 / 4_447_994)   | (52_221 / 54_769)        | (21_030 / 47_218)       | (29x / 81x)                   | (73x / 94x)                 | (-29x / -81x)                 | (3x / 1x)                     | (-73x / -94x)               | (-3x / -1x)                   |
| int32_init_heap: 100      | (2_120_196 / 5_044_091)   | (1_326_292 / 1_328_840)  | (214_542 / 240_749)     | (2x / 4x)                     | (10x / 21x)                 | (-2x / -4x)                   | (6x / 6x)                     | (-10x / -21x)               | (-6x / -6x)                   |
| int64_init_stack: 1       | (1_462_758 / 4_388_682)   | (342 / 2_890)            | (253 / 26_456)          | (4_277x / 1_519x)             | (5_782x / 166x)             | (-4_277x / -1_519x)           | (1x / -9x)                    | (-5_782x / -166x)           | (-1x / 9x)                    |
| int64_init_stack: 10      | (1_474_200 / 4_399_243)   | (846 / 3_394)            | (397 / 27_056)          | (1_743x / 1_296x)             | (3_713x / 163x)             | (-1_743x / -1_296x)           | (2x / -8x)                    | (-3_713x / -163x)           | (-2x / 8x)                    |
| int64_init_stack: 100     | (1_589_255 / 4_510_803)   | (5_886 / 8_434)          | (1_837 / 28_496)        | (270x / 535x)                 | (865x / 158x)               | (-270x / -535x)               | (3x / -3x)                    | (-865x / -158x)             | (-3x / 3x)                    |
| int64_init_heap: 1        | (1_466_907 / 4_389_229)   | (5_269 / 7_817)          | (2_362 / 28_982)        | (278x / 562x)                 | (625x / 151x)               | (-278x / -562x)               | (2x / -4x)                    | (-625x / -151x)             | (-2x / 4x)                    |
| int64_init_heap: 10       | (1_527_802 / 4_450_249)   | (52_316 / 54_864)        | (21_121 / 47_760)       | (29x / 81x)                   | (73x / 93x)                 | (-29x / -81x)                 | (3x / 1x)                     | (-73x / -93x)               | (-3x / -1x)                   |
| int64_init_heap: 100      | (2_127_651 / 5_048_938)   | (1_327_207 / 1_329_755)  | (214_525 / 241_204)     | (2x / 4x)                     | (10x / 21x)                 | (-2x / -4x)                   | (6x / 6x)                     | (-10x / -21x)               | (-6x / -6x)                   |
| nat_init_stack: 1         | (1_462_197 / 4_384_613)   | (342 / 2_890)            | (877 / 27_556)          | (4_275x / 1_517x)             | (1_667x / 159x)             | (-4_275x / -1_517x)           | (-3x / -10x)                  | (-1_667x / -159x)           | (3x / 10x)                    |
| nat_init_stack: 10        | (1_474_111 / 4_397_633)   | (846 / 3_394)            | (3_933 / 30_612)        | (1_742x / 1_296x)             | (375x / 144x)               | (-1_742x / -1_296x)           | (-5x / -9x)                   | (-375x / -144x)             | (5x / 9x)                     |
| nat_init_stack: 100       | (1_591_234 / 4_515_739)   | (5_886 / 8_434)          | (37_188 / 63_867)       | (270x / 535x)                 | (43x / 71x)                 | (-270x / -535x)               | (-6x / -8x)                   | (-43x / -71x)               | (6x / 8x)                     |
| nat_init_heap: 1          | (1_467_764 / 4_393_868)   | (5_221 / 7_769)          | (2_968 / 29_695)        | (281x / 566x)                 | (495x / 148x)               | (-281x / -566x)               | (2x / -4x)                    | (-495x / -148x)             | (-2x / 4x)                    |
| nat_init_heap: 10         | (1_527_103 / 4_449_444)   | (51_945 / 54_493)        | (24_919 / 51_655)       | (29x / 82x)                   | (62x / 86x)                 | (-29x / -82x)                 | (2x / 1x)                     | (-62x / -86x)               | (-2x / -1x)                   |
| nat_init_heap: 100        | (2_124_756 / 5_046_603)   | (1_323_546 / 1_326_094)  | (252_451 / 279_132)     | (2x / 4x)                     | (8x / 18x)                  | (-2x / -4x)                   | (5x / 5x)                     | (-8x / -18x)                | (-5x / -5x)                   |
| nat8_init_stack: 1        | (1_461_004 / 4_382_846)   | (342 / 2_890)            | (255 / 26_914)          | (4_272x / 1_517x)             | (5_729x / 163x)             | (-4_272x / -1_517x)           | (1x / -9x)                    | (-5_729x / -163x)           | (-1x / 9x)                    |
| nat8_init_stack: 10       | (1_475_755 / 4_401_463)   | (846 / 3_394)            | (417 / 27_076)          | (1_744x / 1_297x)             | (3_539x / 163x)             | (-1_744x / -1_297x)           | (2x / -8x)                    | (-3_539x / -163x)           | (-2x / 8x)                    |
| nat8_init_stack: 100      | (1_587_495 / 4_511_700)   | (5_886 / 8_434)          | (2_037 / 28_696)        | (270x / 535x)                 | (779x / 157x)               | (-270x / -535x)               | (3x / -3x)                    | (-779x / -157x)             | (-3x / 3x)                    |
| nat8_init_heap: 1         | (1_468_524 / 4_391_636)   | (5_171 / 7_719)          | (2_357 / 28_977)        | (284x / 569x)                 | (627x / 152x)               | (-284x / -569x)               | (2x / -4x)                    | (-627x / -152x)             | (-2x / 4x)                    |
| nat8_init_heap: 10        | (1_526_852 / 4_451_176)   | (51_702 / 54_250)        | (21_263 / 47_923)       | (30x / 82x)                   | (72x / 93x)                 | (-30x / -82x)                 | (2x / 1x)                     | (-72x / -93x)               | (-2x / -1x)                   |
| nat8_init_heap: 100       | (2_122_436 / 5_044_212)   | (1_321_072 / 1_323_620)  | (215_342 / 241_967)     | (2x / 4x)                     | (10x / 21x)                 | (-2x / -4x)                   | (6x / 5x)                     | (-10x / -21x)               | (-6x / -5x)                   |
| nat16_init_stack: 1       | (1_461_791 / 4_386_251)   | (342 / 2_890)            | (255 / 26_864)          | (4_274x / 1_518x)             | (5_733x / 163x)             | (-4_274x / -1_518x)           | (1x / -9x)                    | (-5_733x / -163x)           | (-1x / 9x)                    |
| nat16_init_stack: 10      | (1_474_861 / 4_397_253)   | (846 / 3_394)            | (417 / 27_026)          | (1_743x / 1_296x)             | (3_537x / 163x)             | (-1_743x / -1_296x)           | (2x / -8x)                    | (-3_537x / -163x)           | (-2x / 8x)                    |
| nat16_init_stack: 100     | (1_584_301 / 4_504_843)   | (5_886 / 8_434)          | (2_037 / 28_646)        | (269x / 534x)                 | (778x / 157x)               | (-269x / -534x)               | (3x / -3x)                    | (-778x / -157x)             | (-3x / 3x)                    |
| nat16_init_heap: 1        | (1_467_385 / 4_392_167)   | (5_171 / 7_719)          | (2_334 / 28_949)        | (284x / 569x)                 | (633x / 152x)               | (-284x / -569x)               | (2x / -4x)                    | (-633x / -152x)             | (-2x / 4x)                    |
| nat16_init_heap: 10       | (1_525_909 / 4_449_616)   | (51_702 / 54_250)        | (21_064 / 47_719)       | (30x / 82x)                   | (73x / 93x)                 | (-30x / -82x)                 | (2x / 1x)                     | (-73x / -93x)               | (-2x / -1x)                   |
| nat16_init_heap: 100      | (2_120_465 / 5_043_867)   | (1_321_078 / 1_323_626)  | (213_832 / 240_475)     | (2x / 4x)                     | (10x / 21x)                 | (-2x / -4x)                   | (6x / 5x)                     | (-10x / -21x)               | (-6x / -5x)                   |
| nat32_init_stack: 1       | (1_464_845 / 4_388_963)   | (342 / 2_890)            | (255 / 26_890)          | (4_283x / 1_519x)             | (5_744x / 163x)             | (-4_283x / -1_519x)           | (1x / -9x)                    | (-5_744x / -163x)           | (-1x / 9x)                    |
| nat32_init_stack: 10      | (1_472_513 / 4_394_370)   | (846 / 3_394)            | (417 / 27_052)          | (1_741x / 1_295x)             | (3_531x / 162x)             | (-1_741x / -1_295x)           | (2x / -8x)                    | (-3_531x / -162x)           | (-2x / 8x)                    |
| nat32_init_stack: 100     | (1_588_707 / 4_513_906)   | (5_886 / 8_434)          | (2_037 / 28_672)        | (270x / 535x)                 | (780x / 157x)               | (-270x / -535x)               | (3x / -3x)                    | (-780x / -157x)             | (-3x / 3x)                    |
| nat32_init_heap: 1        | (1_470_927 / 4_393_490)   | (5_186 / 7_734)          | (2_357 / 29_019)        | (284x / 568x)                 | (628x / 151x)               | (-284x / -568x)               | (2x / -4x)                    | (-628x / -151x)             | (-2x / 4x)                    |
| nat32_init_heap: 10       | (1_528_914 / 4_455_731)   | (51_846 / 54_394)        | (21_584 / 48_249)       | (30x / 82x)                   | (71x / 92x)                 | (-30x / -82x)                 | (2x / 1x)                     | (-71x / -92x)               | (-2x / -1x)                   |
| nat32_init_heap: 100      | (2_120_126 / 5_040_316)   | (1_322_596 / 1_325_144)  | (217_181 / 243_846)     | (2x / 4x)                     | (10x / 21x)                 | (-2x / -4x)                   | (6x / 5x)                     | (-10x / -21x)               | (-6x / -5x)                   |
| nat64_init_stack: 1       | (1_460_748 / 4_380_615)   | (342 / 2_890)            | (253 / 26_912)          | (4_271x / 1_516x)             | (5_774x / 163x)             | (-4_271x / -1_516x)           | (1x / -9x)                    | (-5_774x / -163x)           | (-1x / 9x)                    |
| nat64_init_stack: 10      | (1_472_964 / 4_396_486)   | (846 / 3_394)            | (397 / 27_056)          | (1_741x / 1_295x)             | (3_710x / 162x)             | (-1_741x / -1_295x)           | (2x / -8x)                    | (-3_710x / -162x)           | (-2x / 8x)                    |
| nat64_init_stack: 100     | (1_589_639 / 4_512_326)   | (5_886 / 8_434)          | (1_837 / 28_496)        | (270x / 535x)                 | (865x / 158x)               | (-270x / -535x)               | (3x / -3x)                    | (-865x / -158x)             | (-3x / 3x)                    |
| nat64_init_heap: 1        | (1_466_475 / 4_390_936)   | (5_196 / 7_744)          | (2_362 / 28_982)        | (282x / 567x)                 | (624x / 152x)               | (-282x / -567x)               | (2x / -4x)                    | (-624x / -152x)             | (-2x / 4x)                    |
| nat64_init_heap: 10       | (1_526_314 / 4_448_105)   | (51_946 / 54_494)        | (21_121 / 47_760)       | (29x / 82x)                   | (73x / 93x)                 | (-29x / -82x)                 | (2x / 1x)                     | (-73x / -93x)               | (-2x / -1x)                   |
| nat64_init_heap: 100      | (2_123_052 / 5_047_899)   | (1_323_584 / 1_326_132)  | (214_354 / 241_033)     | (2x / 4x)                     | (10x / 21x)                 | (-2x / -4x)                   | (6x / 5x)                     | (-10x / -21x)               | (-6x / -5x)                   |
| null_init_stack: 1        | (1_463_226 / 4_387_275)   | (342 / 2_890)            | (255 / 26_932)          | (4_278x / 1_518x)             | (5_738x / 163x)             | (-4_278x / -1_518x)           | (1x / -9x)                    | (-5_738x / -163x)           | (-1x / 9x)                    |
| null_init_stack: 10       | (1_473_662 / 4_395_966)   | (846 / 3_394)            | (417 / 27_094)          | (1_742x / 1_295x)             | (3_534x / 162x)             | (-1_742x / -1_295x)           | (2x / -8x)                    | (-3_534x / -162x)           | (-2x / 8x)                    |
| null_init_stack: 100      | (1_586_842 / 4_510_701)   | (5_886 / 8_434)          | (2_037 / 28_714)        | (270x / 535x)                 | (779x / 157x)               | (-270x / -535x)               | (3x / -3x)                    | (-779x / -157x)             | (-3x / 3x)                    |
| null_init_heap: 1         | (1_469_774 / 4_397_723)   | (5_207 / 7_755)          | (2_285 / 28_968)        | (282x / 567x)                 | (645x / 152x)               | (-282x / -567x)               | (2x / -4x)                    | (-645x / -152x)             | (-2x / 4x)                    |
| null_init_heap: 10        | (1_527_517 / 4_451_641)   | (52_062 / 54_610)        | (20_962 / 47_752)       | (29x / 82x)                   | (73x / 93x)                 | (-29x / -82x)                 | (3x / 1x)                     | (-73x / -93x)               | (-3x / -1x)                   |
| null_init_heap: 100       | (2_118_817 / 5_041_485)   | (1_324_670 / 1_327_218)  | (213_207 / 239_884)     | (2x / 4x)                     | (10x / 21x)                 | (-2x / -4x)                   | (6x / 6x)                     | (-10x / -21x)               | (-6x / -6x)                   |
| opt_init_stack: 1         | (1_466_065 / 4_388_219)   | (342 / 2_890)            | (255 / 26_914)          | (4_287x / 1_518x)             | (5_749x / 163x)             | (-4_287x / -1_518x)           | (1x / -9x)                    | (-5_749x / -163x)           | (-1x / 9x)                    |
| opt_init_stack: 10        | (1_494_357 / 4_424_681)   | (846 / 3_394)            | (417 / 27_076)          | (1_766x / 1_304x)             | (3_584x / 163x)             | (-1_766x / -1_304x)           | (2x / -8x)                    | (-3_584x / -163x)           | (-2x / 8x)                    |
| opt_init_stack: 100       | (1_753_051 / 4_679_911)   | (5_886 / 8_434)          | (2_037 / 28_696)        | (298x / 555x)                 | (861x / 163x)               | (-298x / -555x)               | (3x / -3x)                    | (-861x / -163x)             | (-3x / 3x)                    |
| opt_init_heap: 1          | (1_469_454 / 4_392_165)   | (5_186 / 7_734)          | (2_335 / 29_000)        | (283x / 568x)                 | (633x / 151x)               | (-283x / -568x)               | (2x / -4x)                    | (-633x / -151x)             | (-2x / 4x)                    |
| opt_init_heap: 10         | (1_542_346 / 4_463_464)   | (51_948 / 54_496)        | (21_267 / 47_932)       | (30x / 82x)                   | (73x / 93x)                 | (-30x / -82x)                 | (2x / 1x)                     | (-73x / -93x)               | (-2x / -1x)                   |
| opt_init_heap: 100        | (2_281_610 / 5_213_464)   | (1_323_590 / 1_326_138)  | (216_084 / 242_749)     | (2x / 4x)                     | (11x / 22x)                 | (-2x / -4x)                   | (6x / 5x)                     | (-11x / -22x)               | (-6x / -5x)                   |
| principal_init_stack: 1   | (2_058_072 / 4_978_131)   | (6_968 / 9_516)          | (16_942 / 43_603)       | (295x / 523x)                 | (121x / 114x)               | (-295x / -523x)               | (-2x / -5x)                   | (-121x / -114x)             | (2x / 5x)                     |
| principal_init_stack: 10  | (5_757_562 / 8_682_462)   | (45_621 / 48_169)        | (120_921 / 147_582)     | (126x / 180x)                 | (48x / 59x)                 | (-126x / -180x)               | (-3x / -3x)                   | (-48x / -59x)               | (3x / 3x)                     |
| principal_init_stack: 100 | (44_409_675 / 47_339_408) | (453_636 / 456_184)      | (1_207_446 / 1_234_107) | (98x / 104x)                  | (37x / 38x)                 | (-98x / -104x)                | (-3x / -3x)                   | (-37x / -38x)               | (3x / 3x)                     |
| principal_init_heap: 1    | (2_063_790 / 4_985_713)   | (11_847 / 14_395)        | (19_175 / 45_837)       | (174x / 346x)                 | (108x / 109x)               | (-174x / -346x)               | (-2x / -3x)                   | (-108x / -109x)             | (2x / 3x)                     |
| principal_init_heap: 10   | (5_788_976 / 8_705_653)   | (96_966 / 99_514)        | (141_408 / 168_089)     | (60x / 87x)                   | (41x / 52x)                 | (-60x / -87x)                 | (-1x / -2x)                   | (-41x / -52x)               | (1x / 2x)                     |
| principal_init_heap: 100  | (44_717_244 / 47_621_128) | (1_773_775 / 1_776_323)  | (1_414_325 / 1_440_963) | (25x / 27x)                   | (32x / 33x)                 | (-25x / -27x)                 | (1x / 1x)                     | (-32x / -33x)               | (-1x / -1x)                   |
| record_init_stack: 1      | (2_084_725 / 4_980_038)   | (7_471 / 10_019)         | (17_344 / 43_977)       | (279x / 497x)                 | (120x / 113x)               | (-279x / -497x)               | (-2x / -4x)                   | (-120x / -113x)             | (2x / 4x)                     |
| record_init_stack: 10     | (6_161_121 / 9_074_139)   | (50_612 / 53_160)        | (125_768 / 152_401)     | (122x / 171x)                 | (49x / 60x)                 | (-122x / -171x)               | (-2x / -3x)                   | (-49x / -60x)               | (2x / 3x)                     |
| record_init_stack: 100    | (48_549_088 / 51_470_912) | (503_492 / 506_040)      | (1_255_538 / 1_282_171) | (96x / 102x)                  | (39x / 40x)                 | (-96x / -102x)                | (-2x / -3x)                   | (-39x / -40x)               | (2x / 3x)                     |
| record_init_heap: 1       | (2_103_524 / 5_020_929)   | (12_350 / 14_898)        | (19_723 / 46_356)       | (170x / 337x)                 | (107x / 108x)               | (-170x / -337x)               | (-2x / -3x)                   | (-107x / -108x)             | (2x / 3x)                     |
| record_init_heap: 10      | (6_205_901 / 9_116_320)   | (101_951 / 104_499)      | (149_229 / 175_807)     | (61x / 87x)                   | (42x / 52x)                 | (-61x / -87x)                 | (-1x / -2x)                   | (-42x / -52x)               | (1x / 2x)                     |
| record_init_heap: 100     | (48_974_197 / 51_871_616) | (1_823_619 / 1_826_167)  | (1_490_295 / 1_516_903) | (27x / 29x)                   | (33x / 34x)                 | (-27x / -29x)                 | (1x / 1x)                     | (-33x / -34x)               | (-1x / -1x)                   |
| text_init_stack: 1        | (1_447_438 / 4_341_563)   | (342 / 2_890)            | (565 / 27_194)          | (4_232x / 1_502x)             | (2_562x / 160x)             | (-4_232x / -1_502x)           | (-2x / -9x)                   | (-2_562x / -160x)           | (2x / 9x)                     |
| text_init_stack: 10       | (1_460_656 / 4_357_037)   | (846 / 3_394)            | (2_138 / 28_767)        | (1_727x / 1_284x)             | (683x / 151x)               | (-1_727x / -1_284x)           | (-3x / -8x)                   | (-683x / -151x)             | (3x / 8x)                     |
| text_init_stack: 100      | (1_580_283 / 4_479_971)   | (5_886 / 8_434)          | (19_238 / 45_867)       | (268x / 531x)                 | (82x / 98x)                 | (-268x / -531x)               | (-3x / -5x)                   | (-82x / -98x)               | (3x / 5x)                     |
| text_init_heap: 1         | (1_456_982 / 4_357_691)   | (5_221 / 7_769)          | (2_762 / 29_390)        | (279x / 561x)                 | (528x / 148x)               | (-279x / -561x)               | (2x / -4x)                    | (-528x / -148x)             | (-2x / 4x)                    |
| text_init_heap: 10        | (1_517_302 / 4_421_288)   | (52_191 / 54_739)        | (23_391 / 50_053)       | (29x / 81x)                   | (65x / 88x)                 | (-29x / -81x)                 | (2x / 1x)                     | (-65x / -88x)               | (-2x / -1x)                   |
| text_init_heap: 100       | (2_126_212 / 5_046_432)   | (1_326_031 / 1_328_579)  | (234_734 / 261_413)     | (2x / 4x)                     | (9x / 19x)                  | (-2x / -4x)                   | (6x / 5x)                     | (-9x / -19x)                | (-6x / -5x)                   |
| variant_init_stack: 1     | (1_462_202 / 4_386_691)   | (342 / 2_890)            | (255 / 26_903)          | (4_275x / 1_518x)             | (5_734x / 163x)             | (-4_275x / -1_518x)           | (1x / -9x)                    | (-5_734x / -163x)           | (-1x / 9x)                    |
| variant_init_stack: 10    | (1_498_737 / 4_422_951)   | (846 / 3_394)            | (417 / 27_065)          | (1_772x / 1_303x)             | (3_594x / 163x)             | (-1_772x / -1_303x)           | (2x / -8x)                    | (-3_594x / -163x)           | (-2x / 8x)                    |
| variant_init_stack: 100   | (1_843_676 / 4_767_168)   | (5_886 / 8_434)          | (2_037 / 28_685)        | (313x / 565x)                 | (905x / 166x)               | (-313x / -565x)               | (3x / -3x)                    | (-905x / -166x)             | (-3x / 3x)                    |
| variant_init_heap: 1      | (1_471_315 / 4_394_499)   | (5_221 / 7_769)          | (2_338 / 29_187)        | (282x / 566x)                 | (630x / 151x)               | (-282x / -566x)               | (2x / -4x)                    | (-630x / -151x)             | (-2x / 4x)                    |
| variant_init_heap: 10     | (1_543_458 / 4_463_978)   | (52_191 / 54_739)        | (21_767 / 48_401)       | (30x / 82x)                   | (72x / 93x)                 | (-30x / -82x)                 | (2x / 1x)                     | (-72x / -93x)               | (-2x / -1x)                   |
| variant_init_heap: 100    | (2_275_885 / 5_199_537)   | (1_326_025 / 1_328_573)  | (217_056 / 243_687)     | (2x / 4x)                     | (11x / 21x)                 | (-2x / -4x)                   | (6x / 5x)                     | (-11x / -21x)               | (-6x / -5x)                   |
| vec_init_stack: 1         | (1_470_144 / 4_394_696)   | (342 / 2_890)            | (494 / 27_123)          | (4_299x / 1_521x)             | (2_976x / 162x)             | (-4_299x / -1_521x)           | (-1x / -9x)                   | (-2_976x / -162x)           | (1x / 9x)                     |
| vec_init_stack: 10        | (1_517_253 / 4_441_175)   | (846 / 3_394)            | (1_763 / 28_392)        | (1_793x / 1_309x)             | (861x / 156x)               | (-1_793x / -1_309x)           | (-2x / -8x)                   | (-861x / -156x)             | (2x / 8x)                     |
| vec_init_stack: 100       | (2_016_823 / 4_941_439)   | (5_886 / 8_434)          | (15_488 / 42_117)       | (343x / 586x)                 | (130x / 117x)               | (-343x / -586x)               | (-3x / -5x)                   | (-130x / -117x)             | (3x / 5x)                     |
| vec_init_heap: 1          | (1_477_087 / 4_399_022)   | (5_221 / 7_769)          | (2_667 / 29_280)        | (283x / 566x)                 | (555x / 150x)               | (-283x / -566x)               | (2x / -4x)                    | (-555x / -150x)             | (-2x / 4x)                    |
| vec_init_heap: 10         | (1_576_838 / 4_503_076)   | (52_191 / 54_739)        | (22_881 / 49_479)       | (30x / 82x)                   | (69x / 91x)                 | (-30x / -82x)                 | (2x / 1x)                     | (-69x / -91x)               | (-2x / -1x)                   |
| vec_init_heap: 100        | (2_574_760 / 5_497_476)   | (1_326_025 / 1_328_573)  | (232_622 / 259_261)     | (2x / 4x)                     | (11x / 21x)                 | (-2x / -4x)                   | (6x / 5x)                     | (-11x / -21x)               | (-6x / -5x)                   |
